class FiscalIncrementException(Exception):
    pass


class FiscalGetIdException(Exception):
    pass


class FiscalParametersException(Exception):
    pass


class FiscalValidation(Exception):
    pass
