from production.model import Course
from production.repository import ProductRepository
from typing import List


class GetCoursesInteractor(object):
    def __init__(self, product_repository):
        # type: (ProductRepository) -> None

        self.product_repository = product_repository

    def execute(self):
        # type: () -> List[Course]

        return self.product_repository.get_all_courses()
