from _UpdateGoalsProcessor import UpdateGoalsProcessor
from _UpdateTotalSoldProcessor import UpdateTotalSoldProcessor
from _UpdateTotalSoldOrderProcessor import UpdateTotalSoldOrderProcessor
from _UpdateTotalSoldTableProcessor import UpdateTotalSoldTableProcessor
from _GetSalesChallengeProcessor import GetSalesChallengeProcessor
from _InsertSalesChallengeProcessor import InsertSalesChallengeProcessor
from _DeleteSalesChallengeProcessor import DeleteSalesChallengeProcessor
from _UpdateSalesChallengeProcessor import UpdateSalesChallengeProcessor
