def get_operator_info(model, operator_id):
    need_logged = True
    session = ''
    for op in model.find('Operators').findall('Operator'):
        if op.get('id') == operator_id:
            operator = op
            session = operator.get('sessionId')
            operator_id = operator.get("id")
            if op.get('state') == 'PAUSED':
                need_logged = False
            break

    return need_logged, operator_id, session
