from datetime import datetime


class DeviceStatusHistory(object):
    def __init__(
        self,
        enabled,    # type: bool
        operator,   # type: str
        timestamp,  # type: datetime
    ):
        # type: (...) -> None

        self.timestamp = timestamp
        self.operator = operator
        self.enabled = enabled
